﻿function GalaxyHistoryGadget() {
	this.request = null;
	this.lastIndex = 0;
	this.foo = Math.floor(Math.random() * 1000 * 1000 * 1000);

	this.messages = new Object();
}

GalaxyHistoryGadget.prototype.sendRequest = function() {
	var userName = options.getValue(SK_USER_NAME);
	if (!userName) {
		userName = "";
	}
	var baseURL = options.getValue(SK_BASE_URL);
	if (!baseURL) {
		baseURL = BASE_URL;
	}

	this.request = new XMLHttpRequest();
	try {
		this.foo++;
		var url = (baseURL + HISTORY_PATH + "?u=" + encodeURIComponent(userName)
				+"&i=" + this.lastIndex + "&foo=" + this.foo);
		this.request.open("GET", url, true);
	} catch (e) {
		list.visible = false;
		statusLabel.visible = true;
		statusLabel.innerText = CON_PROBLEM;
		debug.error(e);
		this.request = null;
		return;
	}
	list.visible = true;
	statusLabel.visible = false;

	var self = this;
	this.request.onreadystatechange = function() {
			self.onData();
		};

	try {
		this.request.send();
	} catch (e) {
		list.visible = false;
		statusLabel.visible = true;
		statusLabel.innerText = IO_PROBLEM;
		debug.error(e);
		this.request = null;
		return;
	}
	list.visible = true;
	statusLabel.visible = false;
};

GalaxyHistoryGadget.prototype.onData = function() {
	if (!this.request || !this.request.readyState || (this.request.readyState != 4)) {
		return;
	}
	if (!this.request.status || (this.request.status != 200)) {
		list.visible = false;
		statusLabel.visible = true;
		statusLabel.innerText = IO_STATUS_PROBLEM;
		debug.error("Invalid response status!");
		this.request = null;
		return;
	}
	list.visible = true;
	statusLabel.visible = false;

	var text = this.request.responseText;
	if (text) {
		var parser = new GalaxyHistoryParser();
		this.newMessages = parser.parseText(text);
		if (this.newMessages.length > 0) {
			this.newMessages.sort(function(a, b) {
					return (a.index - b.index);
				});
			for (var i in this.newMessages) {
				var newMessage = this.newMessages[i];
				this.lastIndex = Math.max(newMessage.index, this.lastIndex);
				this.messages["" + newMessage.index] = newMessage;
			}
			this.update();
		}
	}

  this.request = null;
};

GalaxyHistoryGadget.prototype.update = function() {
	for (var i in this.newMessages) {
		var newMessage = this.newMessages[i];
		if (this.lastChildren) {
			this.lastChildren = list.insertElement(GalaxyHistoryGadget.formatMessageForList(newMessage), this.lastChildren);
		} else {
			this.lastChildren = list.appendElement(GalaxyHistoryGadget.formatMessageForList(newMessage));
		}
	}
};

// START Handlers

GalaxyHistoryGadget.prototype.onListItemSelect = function() {
	list.selectedItem.children(0).bold = false;
	list.selectedItem.children(1).bold = false;
	list.selectedItem.children(2).bold = false;

	var message = this.messages[list.selectedItem.name];

	var visitedIndices = options.getValue(SK_VISITED_INDICES);
	if (!visitedIndices) {
		visitedIndices = "";
	}
	if (visitedIndices.indexOf("|" + message.index + "|") == -1) {
		visitedIndices += ("|" + message.index + "|");
		if (visitedIndices.length > (MAX_MESSAGE_COUNT * 10)) {
			visitedIndices = visitedIndices.substring(visitedIndices.indexOf("|", 1) + 1);
		}
	}
	options.putValue(SK_VISITED_INDICES, visitedIndices);

	var details = new DetailsView();
  details.html_content = true;
  details.setContent("", undefined, DETAILS_HTML_START
			+ GalaxyHistoryGadget.formatMessageForDetails(message)
			+ DETAILS_HTML_END, false, 0);

	var self = this;
	pluginHelper.showDetailsView(details, "", gddDetailsViewFlagToolbarOpen,
			function(detailsViewFlags) {self.onDetailsViewFeedback(detailsViewFlags)});
};

GalaxyHistoryGadget.prototype.onDetailsViewFeedback = function(detailsViewFlags) {
	if (detailsViewFlags == gddDetailsViewFlagNone) {
		// User closed the details view
	} else if (detailsViewFlags == gddDetailsViewFlagToolbarOpen) {
		// User clicked on the title of the details view
	}
};

GalaxyHistoryGadget.prototype.onOpen = function() {
	var self = this;
	view.setInterval(function() {
			self.sendRequest();
		}, UPDATE_PERIOD_MILLIS);
	this.sendRequest();
};

// END Handlers

GalaxyHistoryGadget.formatMessageForList = function(message) {
	var userName = options.getValue(SK_USER_NAME);
	if (!userName) {
		userName = "";
	}

	var warn = (userName && (message.text.toLowerCase().indexOf("#warn " + userName.toLowerCase()) == 0));
	var visitedIndices = options.getValue(SK_VISITED_INDICES);
	var visited = (visitedIndices && (visitedIndices.indexOf("|" + message.index + "|") != -1));

	var result = "<listitem name=\"" + message.index +"\" cursor=\"hand\">";

	var date = new Date(message.timeMillis);
	result += "<label color=\"" + (warn ? "#FE5218" : "#000000") + "\" width=\"100%\" height=\"14\" size=\"8\"" + (visited ? ">" : " bold=\"true\">");
	result += date.toLocaleTimeString();
	result += "</label><label color=\"#BB1306\" y=\"14\" width=\"100%\" height=\"14\" size=\"8\"" + (visited ? ">" : " bold=\"true\">");
	result += ((message.userName && (message.userName != " ")) ? message.userName.escapeXML() : MESSAGE_EMPTY_USER_NAME);
	result += "</label><label color=\"" + (warn ? "#FE5218" : "#000000") + "\" y=\"28\" width=\"100%\" height=\"28\" size=\"8\" wordWrap=\"true\"" + (visited ? ">" : " bold=\"true\">");
	result += message.textForList;
	result += "</label></listitem>";

	return result;
};

GalaxyHistoryGadget.formatMessageForDetails = function(message) {
	var userName = options.getValue(SK_USER_NAME);
	if (!userName) {
		userName = "";
	}

	var warn = (userName && (message.text.toLowerCase().indexOf("#warn " + userName.toLowerCase()) == 0));

	var result = "<span>";

	var date = new Date(message.timeMillis);
	result += "<span style=\"color: " + (warn ? "#FE5218" : "#000000") + "\">";
	result += date.toLocaleTimeString();
	result += "</span>&nbsp;<span style=\"color: #BB1306\">";
	result += (message.userName ? message.userName.escapeXML() : " ");
	result += "</span>&nbsp;<span  style=\"color: " + (warn ? "#FE5218" : "#000000") + "\">";
	result += message.html;
	result += "</span>";

	return result;
};

var galaxyHistoryGadget = new GalaxyHistoryGadget();